﻿public class RepoStatistics
{
    public IList<RepoAndDirectory> GetRepoAndDirectories(string rootDirectory)
    {
        string[] repositories = Directory.GetDirectories(rootDirectory, "*.git", SearchOption.AllDirectories);
        var repositoriesAndDirectories = new List<RepoAndDirectory>();
        foreach (var repository in repositories)
        {
            string[] lines = File.ReadAllLines(repository + Path.DirectorySeparatorChar + "config");
            foreach (var line in lines)
            {
                if (line.Contains("url"))
                {
                    repositoriesAndDirectories.Add(new RepoAndDirectory { RepoName = line.Substring(7), DirectoryPath = repository.Substring(0, repository.Length - 5) });
                    break;
                }
            }
        }
        return repositoriesAndDirectories;
    }

    public Dictionary<string, FileResult> ProcessDirectory(string directoryPath, string[] fileTypesToSearchFor)
    {
        var directoryResults = new Dictionary<string, FileResult>();
        var fileNames = Directory.GetFiles(directoryPath, "*.*", SearchOption.AllDirectories)
            .Where(file => fileTypesToSearchFor.Contains(Path.GetExtension(file).ToLower())) // match .CS files too
            .ToList();

        foreach (string fileName in fileNames)
        {
            if(fileName.Contains("artifacts") || fileName.Contains(Path.DirectorySeparatorChar + "bin" + Path.DirectorySeparatorChar) || fileName.Contains(Path.DirectorySeparatorChar + "obj" + Path.DirectorySeparatorChar))
            {
                continue;
            }
            string extension = Path.GetExtension(fileName).ToLower();
            FileResult resultForFileType = directoryResults.GetValueOrDefault(extension);
            if (resultForFileType == null)
            {
                resultForFileType = new FileResult();
                directoryResults.Add(extension, resultForFileType);
            }
            AnalyzeFile(fileName, resultForFileType);
        }

        return directoryResults;
    }

    private void AnalyzeFile(string fileName, FileResult fileResult)
    {
        string extension = Path.GetExtension(fileName).ToLower();

        var lines = File.ReadAllLines(fileName);

        int totalLinesInFile = lines.Length + 1;
        int blankLinesInFile = 0;
        int braceLinesInFile = 0;
        int commentLinesInFile = 0;

        bool inCommentBlock = false;

        foreach (var line in lines)
        {
            string trimmedLine = line.Trim();
            if (extension == ".cs") // only look for comments in .cs files
            {
                if (trimmedLine.Contains("/*") && trimmedLine.Contains("*/"))
                {
                    commentLinesInFile++;
                    continue;
                }
                if (inCommentBlock)
                {
                    if (trimmedLine.Contains("*/"))
                    {
                        inCommentBlock = false;
                    }
                    commentLinesInFile++;
                    continue;
                }
                if (trimmedLine.Contains("/*"))
                {
                    inCommentBlock = true;
                    commentLinesInFile++;
                    continue;
                }

                if (trimmedLine.StartsWith("//"))
                {
                    commentLinesInFile++;
                    continue;
                }
            }

            if (string.IsNullOrWhiteSpace(trimmedLine))
            {
                blankLinesInFile++;
                continue;
            }
            if (extension == ".cs") // only look for brace lines in .cs files
            {
                if (trimmedLine == "{" || trimmedLine == "}")
                {
                    braceLinesInFile++;
                    continue;
                }
            }
        }
        fileResult.TotalLines += totalLinesInFile;
        fileResult.BlankLines += blankLinesInFile;
        fileResult.BraceLines += braceLinesInFile;
        fileResult.CommentedLines += commentLinesInFile;
        fileResult.FileCount++;
    }
}